{ confi8, lib, modulesPath, home-manager, ... } @ inputs :
let
pkgs = inputs.nixpkgs.outputs.legacyPackages.x86_64-linux;
edge = inputs.edge.outputs.legacyPackages.x86_64-linux;
nushell = edge.nushell;
render-params = params: lib.concatMapStringsSep " " (param: ''"${param}"'') params;

basic = rec {
  shape = {
    nix = ''pane_template name="nix" { command "nix"; start_suspended true; };'';
  };

  # change = ''pane command="hx" borderless=true focus=true'';
  scope = nu "scope .";
  change = ''pane command="nvim" borderless=true focus=true { args "-c" "Telescope git_files"; };'';
  under = ''pane size=1 borderless=true { plugin location="zellij:status-bar"; };'';
  upper = ''pane size=1 borderless=true { plugin location="zellij:tab-bar"; };'';
  sudo = command: ''pane command="sudo" { args ${render-params command}; };'';
  nu = command: ''pane command="${nushell}/bin/nu" { args "-e" "${command}"; };'';
  nup = command: ''pane command="${nushell}/bin/nu" { args "-e" "${command}"; start_suspended true; };'';
  ready = command: ''pane command="${nushell}/bin/nu" { args "-e" "${command}"; start_suspended true; };'';

  ns = pkgs: command: ''pane command="nix-shell" { args "-p" "${
  lib.concatStringsSep ''" "'' pkgs}" "${
  lib.replaceStrings [" "] [''" "''] command}";
  start_suspended true; }'';

  nd = command: ''pane command="nix" { args "develop" "--command" "${
  lib.replaceStrings [" "] [''" "''] command}";
  start_suspended true; };'';

  column = inner: ''pane split_direction="horizontal" {
  ${inner}
  };'';

  banner = inner: ''pane split_direction="vertical" {
  ${inner}
  };'';
};

plans = {
  # oh no! lib is less-used, so should slide behind `basic` in call sequence.
  gleam = (import ../session/gleam.nix basic);
  snake = (import ../session/snake.nix basic);
  slice = (import ../session/slice.nix basic);
  sip = (import ../session/sip.nix basic);
  bird = (import ../session/bird.nix lib basic);
  exercise = (import ../session/exercise.nix basic);
};

# remember sessions using a symlink, `~/session/${name}`
# home-manager.users.calliope.home.file."session/${name}" = '''';
build_session = { name, plan, lines ? [] }: {
  home-manager.users.calliope.home.file.".config/zellij/session/${name}.kdl".text = plan {
    base = name;
    inherit lines;
  };
};

in {
  home-manager.users.calliope.home.packages = with pkgs; [ zellij ];

  # enough is enough. come back and clean up more soon.
  imports = [
    (build_session { name = "blade"; plan = plans.slice; })
    (build_session { name = "medi"; plan = plans.snake; })
    (build_session { name = "od"; plan = plans.bird; })
    (build_session { name = "operand"; plan = plans.bird; })
    (build_session { name = "panel"; plan = plans.gleam; })
    (build_session { name = "lens"; plan = plans.gleam; })
    (build_session { name = "press"; plan = plans.bird; })
    (build_session { name = "cc"; plan = plans.bird; })
    (build_session { name = "pain"; plan = plans.bird; })
  ];

  home-manager.users.calliope.home.file.".config/zellij/session/home.kdl".text = (''
    layout { pane; }
  '');

  home-manager.users.calliope.home.file.".config/zellij/session/nu.kdl".text = (''
    layout {
    ${basic.upper}
    cwd "~/disc/nvme0n1p5/rebase.hub/nushell/nushell"; pane;
    ${basic.under}
    }
  '');


  # home-manager.users.calliope.home.file.".config/zellij/session/hac.kdl".text = (build_session { name = "hac"; plan = plans.bird; lines = [
  #   (basic.sudo ["${nushell}/bin/nu" "-e" "nix run .#wp"])
  #   (basic.sudo ["${nushell}/bin/nu" "-e" "kill (ps | where name =~ 'micro' | get pid | get 0)"])
  # ]; });

  home-manager.users.calliope.home.file.".config/zellij/session/share.kdl".text = ''
    layout {
      cwd "~/base/code/share";
      ${basic.column ''
        ${basic.change}
        pane
        pane command="git" { args "push" "deploy" "main"; start_suspended true; }
      ''}
    }
  '';

  home-manager.users.calliope.home.file.".config/zellij/session/qu.kdl".text = ''
    layout {
      cwd "~/qu/";
      ${basic.column ''${basic.scope};
      pane;
    }''} }'';

  home-manager.users.calliope.home.file.".config/zellij/session/signal.kdl".text = ''
    layout { cwd "~";
    ${basic.column ''
      ${basic.scope}
      ${basic.ns [ "wireshark" ] "wireshark"}
    ''}
  }'';

  home-manager.users.calliope.home.file.".config/zellij/session/build.kdl".text = ''
    layout { cwd "~/.build/nixos";
      ${basic.column ''
        ${basic.banner ''
          pane command="sudo" { args "${nushell}/bin/nu" "-e" "nixos-rebuild switch"; }
          // pane command="colmena" { args "build"; start_suspended true; }
          ${basic.nup "colmena apply --on base --build-on-target switch --impure"}
          ${basic.nup "shell $m.base run-pain"}
          ${basic.nup "shell $m.base run-operand"}
          ${basic.nup "shell $m.base run-cc"}
        ''}
        ${basic.change}''}
    };'';

  home-manager.users.calliope.home.file.".config/zellij/session/image.kdl".text = ''
    layout { cwd "~/disk";

      ${basic.banner ''

        ${basic.column ''
          pane command="${nushell}/bin/nu" { args "-e" "disk ls"; };
          pane command="${nushell}/bin/nu" { args "-e" "ls"; };
        ''}

        ${basic.column ''
          ${basic.nu "echo hi"}
          ${basic.nu "echo all"}
          ${basic.nu "echo you"}
          ${basic.nu "echo people"}
        ''}

      ''}

    }
  '';

  home-manager.users.calliope.home.file.".config/zellij/session/issue.kdl".text = ''
    layout { cwd "~/issue"; pane command="${nushell}/bin/nu" { args "-e" "issue"; }; }
  '';

  home-manager.users.calliope.home.file.".config/zellij/session/muse.kdl".text = ''
    layout { ${basic.ns [ "spotify-player" ] "spotify_player" }; }
  '';

  home-manager.users.calliope.home.file.".config/zellij/session/record.kdl".text = ''
    layout {
      cwd "~/rec";
      pane command="${nushell}/bin/nu" { args "-e" "ls"; };
      ${basic.ready "screen record"}
    }

  '';

  home-manager.users.calliope.home.file.".config/zellij/config.kdl".text = ''
    // If you'd like to override the default keybindings completely, be sure to change "keybinds" to "keybinds clear-defaults=true"
    keybinds {
      unbind "Ctrl h"
        normal {
          bind "Ctrl m" { SwitchToMode "Move"; }
        }
      locked {
        bind "Ctrl g" { SwitchToMode "Normal"; }
      }
      resize {
        bind "Ctrl n" { SwitchToMode "Normal"; }
        bind "h" "Left" { Resize "Increase Left"; }
        bind "j" "Down" { Resize "Increase Down"; }
        bind "k" "Up" { Resize "Increase Up"; }
        bind "l" "Right" { Resize "Increase Right"; }
        bind "H" { Resize "Decrease Left"; }
        bind "J" { Resize "Decrease Down"; }
        bind "K" { Resize "Decrease Up"; }
        bind "L" { Resize "Decrease Right"; }
        bind "=" "+" { Resize "Increase"; }
        bind "-" { Resize "Decrease"; }
      }
      pane {
        bind "Ctrl p" { SwitchToMode "Normal"; }
        bind "h" "Left" { MoveFocus "Left"; }
        bind "l" "Right" { MoveFocus "Right"; }
        bind "j" "Down" { MoveFocus "Down"; }
        bind "k" "Up" { MoveFocus "Up"; }
        bind "p" { SwitchFocus; }
        bind "n" { NewPane; SwitchToMode "Normal"; }
        bind "d" { NewPane "Down"; SwitchToMode "Normal"; }
        bind "r" { NewPane "Right"; SwitchToMode "Normal"; }
        bind "x" { CloseFocus; SwitchToMode "Normal"; }
        bind "f" { ToggleFocusFullscreen; SwitchToMode "Normal"; }
        bind "z" { TogglePaneFrames; SwitchToMode "Normal"; }
        bind "w" { ToggleFloatingPanes; SwitchToMode "Normal"; }
        bind "e" { TogglePaneEmbedOrFloating; SwitchToMode "Normal"; }
        bind "c" { SwitchToMode "RenamePane"; PaneNameInput 0;}
      }
      move {
        bind "Ctrl m" { SwitchToMode "Normal"; }
        bind "n" "Tab" { MovePane; }
        bind "p" { MovePaneBackwards; }
        bind "h" "Left" { MovePane "Left"; }
        bind "j" "Down" { MovePane "Down"; }
        bind "k" "Up" { MovePane "Up"; }
        bind "l" "Right" { MovePane "Right"; }
      }
      tab {
        bind "Ctrl t" { SwitchToMode "Normal"; }
        bind "r" { SwitchToMode "RenameTab"; TabNameInput 0; }
        bind "h" "Left" "Up" "k" { GoToPreviousTab; }
        bind "l" "Right" "Down" "j" { GoToNextTab; }
        bind "n" { NewTab; SwitchToMode "Normal"; }
        bind "x" { CloseTab; SwitchToMode "Normal"; }
        bind "s" { ToggleActiveSyncTab; SwitchToMode "Normal"; }
        bind "b" { BreakPane; SwitchToMode "Normal"; }
        bind "]" { BreakPaneRight; SwitchToMode "Normal"; }
        bind "[" { BreakPaneLeft; SwitchToMode "Normal"; }
        bind "1" { GoToTab 1; SwitchToMode "Normal"; }
        bind "2" { GoToTab 2; SwitchToMode "Normal"; }
        bind "3" { GoToTab 3; SwitchToMode "Normal"; }
        bind "4" { GoToTab 4; SwitchToMode "Normal"; }
        bind "5" { GoToTab 5; SwitchToMode "Normal"; }
        bind "6" { GoToTab 6; SwitchToMode "Normal"; }
        bind "7" { GoToTab 7; SwitchToMode "Normal"; }
        bind "8" { GoToTab 8; SwitchToMode "Normal"; }
        bind "9" { GoToTab 9; SwitchToMode "Normal"; }
        bind "Tab" { ToggleTab; }
      }
      scroll {
        bind "Ctrl s" { SwitchToMode "Normal"; }
        bind "e" { EditScrollback; SwitchToMode "Normal"; }
        bind "s" { SwitchToMode "EnterSearch"; SearchInput 0; }
        bind "Ctrl c" { ScrollToBottom; SwitchToMode "Normal"; }
        bind "j" "Down" { ScrollDown; }
        bind "k" "Up" { ScrollUp; }
        bind "Ctrl f" "PageDown" "Right" "l" { PageScrollDown; }
        bind "Ctrl b" "PageUp" "Left" "h" { PageScrollUp; }
        bind "d" { HalfPageScrollDown; }
        bind "u" { HalfPageScrollUp; }
        // uncomment this and adjust key if using copy_on_select=false
        // bind "Alt c" { Copy; }
      }
      search {
        bind "Ctrl s" { SwitchToMode "Normal"; }
        bind "Ctrl c" { ScrollToBottom; SwitchToMode "Normal"; }
        bind "j" "Down" { ScrollDown; }
        bind "k" "Up" { ScrollUp; }
        bind "Ctrl f" "PageDown" "Right" "l" { PageScrollDown; }
        bind "Ctrl b" "PageUp" "Left" "h" { PageScrollUp; }
        bind "d" { HalfPageScrollDown; }
        bind "u" { HalfPageScrollUp; }
        bind "n" { Search "down"; }
        bind "p" { Search "up"; }
        bind "c" { SearchToggleOption "CaseSensitivity"; }
        bind "w" { SearchToggleOption "Wrap"; }
        bind "o" { SearchToggleOption "WholeWord"; }
      }
      entersearch {
        bind "Ctrl c" "Esc" { SwitchToMode "Scroll"; }
        bind "Enter" { SwitchToMode "Search"; }
      }
      renametab {
        bind "Ctrl c" { SwitchToMode "Normal"; }
        bind "Esc" { UndoRenameTab; SwitchToMode "Tab"; }
      }
      renamepane {
        bind "Ctrl c" { SwitchToMode "Normal"; }
        bind "Esc" { UndoRenamePane; SwitchToMode "Pane"; }
      }
      session {
        bind "Ctrl o" { SwitchToMode "Normal"; }
        bind "Ctrl s" { SwitchToMode "Scroll"; }
        bind "d" { Detach; }
        bind "w" {
          LaunchOrFocusPlugin "zellij:session-manager" {
            floating true
              move_to_focused_tab true
          };
          SwitchToMode "Normal"
        }
      }
      shared_except "locked" {
        bind "Ctrl g" { SwitchToMode "Locked"; }
        bind "Ctrl q" { Quit; }
        bind "Alt n" { NewPane; }
        bind "Alt h" "Alt Left" { MoveFocusOrTab "Left"; }
        bind "Alt l" "Alt Right" { MoveFocusOrTab "Right"; }
        bind "Alt j" "Alt Down" { MoveFocus "Down"; }
        bind "Alt k" "Alt Up" { MoveFocus "Up"; }
        bind "Alt =" "Alt +" { Resize "Increase"; }
        bind "Alt -" { Resize "Decrease"; }
        bind "Alt [" { PreviousSwapLayout; }
        bind "Alt ]" { NextSwapLayout; }
      }
      shared_except "normal" "locked" {
        bind "Enter" "Esc" { SwitchToMode "Normal"; }
      }
      shared_except "pane" "locked" {
        bind "Ctrl p" { SwitchToMode "Pane"; }
      }
      shared_except "resize" "locked" {
        bind "Ctrl n" { SwitchToMode "Resize"; }
      }
      shared_except "scroll" "locked" {
        bind "Ctrl s" { SwitchToMode "Scroll"; }
      }
      shared_except "session" "locked" {
        bind "Ctrl o" { SwitchToMode "Session"; }
      }
      shared_except "tab" "locked" {
        bind "Ctrl t" { SwitchToMode "Tab"; }
      }
      shared_except "move" "locked" {
        bind "Ctrl m" { SwitchToMode "Move"; }
      }
    }

  plugins {
    tab-bar { path "tab-bar"; }
    status-bar { path "status-bar"; }
    strider { path "strider"; }
    compact-bar { path "compact-bar"; }
    session-manager { path "session-manager"; }
  }

  scrollback_editor "nvim"
    layout_dir "/home/calliope/.config/zellij/session/"
    pane_frames false
    '';
}

# More zellij `config.kdl` options:

# // Choose what to do when zellij receives SIGTERM, SIGINT, SIGQUIT or SIGHUP
# // eg. when terminal window with an active zellij session is closed
# // Options:
# //   - detach (Default)
# //   - quit
# //
# // on_force_close "quit"

# // Send a request for a simplified ui (without arrow fonts) to plugins
# // simplified_ui true

# // Choose the path to the default shell that zellij will use for opening new panes
# // Default: $SHELL
# // default_shell "${nushell}/bin/nu"

# // Choose the path to override cwd that zellij will use for opening new panes
# //
# // default_cwd ""

# // Toggle between having Zellij lay out panes according to a predefined set of layouts whenever possible
# // auto_layout true

# // Whether sessions should be serialized to the cache folder (including their tabs/panes, cwds and running commands) so that they can later be resurrected
# // session_serialization false

# // Whether pane viewports are serialized along with the session, default is false
# // serialize_pane_viewport true

# // Scrollback lines to serialize along with the pane viewport when serializing sessions, 0
# // defaults to the scrollback size. If this number is higher than the scrollback size, it will
# // also default to the scrollback size. This does nothing if `serialize_pane_viewport` is not true.
# //
# // scrollback_lines_to_serialize 10000

# // Define color themes for Zellij
# // For more examples, see: https://github.com/zellij-org/zellij/tree/main/example/themes
# // Once these themes are defined, one of them should to be selected in the "theme" section of this file
# //
# // themes {
# //     dracula {
# //         fg 248 248 242
# //         bg 40 42 54
# //         red 255 85 85
# //         green 80 250 123
# //         yellow 241 250 140
# //         blue 98 114 164
# //         magenta 255 121 198
# //         orange 255 184 108
# //         cyan 139 233 253
# //         black 0 0 0
# //         white 255 255 255
# //     }
# // }

# // Choose the theme that is specified in the themes section.
# // Default: default
# //
# // theme "default"

# // The name of the default layout to load on startup
# // Default: "default"
# // default_layout "compact"

# // Choose the mode that zellij uses when starting up.
# // Default: normal
# //
# // default_mode "locked"

# // Toggle enabling the mouse mode.
# // On certain configurations, or terminals this could
# // potentially interfere with copying text.
# // Options:
# //   - true (default)
# //   - false
# //
# // mouse_mode false

# // Configure the scroll back buffer size
# // This is the number of lines zellij stores for each pane in the scroll back
# // buffer. Excess number of lines are discarded in a FIFO fashion.
# // Valid values: positive integers
# // Default value: 10000
# //
# // scroll_buffer_size 10000

# // Provide a command to execute when copying text. The text will be piped to
# // the stdin of the program to perform the copy. This can be used with
# // terminal emulators which do not support the OSC 52 ANSI control sequence
# // that will be used by default if this option is not set.
# // Examples:
# //
# // copy_command "xclip -selection clipboard" // x11
# // copy_command "wl-copy"                    // wayland
# // copy_command "pbcopy"                     // osx

# // Choose the destination for copied text
# // Allows using the primary selection buffer (on x11/wayland) instead of the system clipboard.
# // Does not apply when using copy_command.
# // Options:
# //   - system (default)
# //   - primary
# //
# // copy_clipboard "primary"

# // Enable or disable automatic copy (and clear) of selection when releasing mouse
# // Default: true
# //
# // copy_on_select false

# // When attaching to an existing session with other users,
# // should the session be mirrored (true)
# // or should each user have their own cursor (false)
# // Default: false
# //
# // mirror_session true

# // The folder in which Zellij will look for themes
# //
# // theme_dir "/path/to/my/theme_dir"

# // Enable or disable the rendering of styled and colored underlines (undercurl).
# // May need to be disabled for certain unsupported terminals
# // Default: true
# //
# // styled_underlines false
